#include <stdlib.h>
#include <ti/drivers/rf/RF.h>
#include <ti/drivers/PIN.h>
#include "Board.h"
#include "RFQueue.h"
#include <smartrf_settings/smartrf_settings.h>
#include <smartrf_settings/smartrf_settings_predefined.h>

/***** Defines *****/
#define NUM_DATA_ENTRIES        1
#define NUM_APPENDED_BYTES      0   // -- For the HS command (6 bytes total):  packet length (2 bytes) + Timestamp (4 bytes)
                                    // -- For other Sub-1 GHz commands (6 bytes total): packet length (1 bytes) + Timestamp (4 bytes) + status (1 byte)
                                    // -- For 200 Kbps IEEE 802.15.4g commands (5 bytes total): Timestamp (4 bytes) + status (1 byte)
                                    // -- For BLE (9 bytes total), but max payload is well under the max length supported by the sub-1 phys: Timestamp (4 bytes) + status (1 byte) + RSSI (1 byte) + CRC (3 bytes)
/***** Prototypes *****/
static void tx_callback(RF_Handle h, RF_CmdHandle ch, RF_EventMask e);

/***** Variable declarations *****/
static RF_Object rfObject;
static RF_Handle rfHandle;

static volatile RF_CmdHandle cmdHandle;

static uint8_t txDataEntryBuffer[RF_QUEUE_DATA_ENTRY_BUFFER_SIZE(NUM_DATA_ENTRIES,
                                                                 MAX_LENGTH,
                                                                 NUM_APPENDED_BYTES)];

/* TX queue or RF Core to read data from */
static dataQueue_t  txDataQueue;
static rfc_dataEntryGeneral_t* tx_currentDataEntry;
static uint8_t *pPacket;

/* Runs the transmitting part of the test application and returns a result. */
void  RF_tran_init (uint16_t  freq_MHz,  uint8_t  N_dBm)
{
    RF_Params rfParams;
    RF_Params_init(&rfParams);

    if( RFQueue_defineQueue(&txDataQueue,
                            txDataEntryBuffer,
                            sizeof(txDataEntryBuffer),
                            NUM_DATA_ENTRIES,
                            MAX_LENGTH + NUM_APPENDED_BYTES)  )  {
        //while(true);  // Failed to allocate space for all data entries
    }

    RF_cmdTxHS.pQueue = &txDataQueue;
    RF_cmdTxHS.startTrigger.triggerType = TRIG_NOW; //TRIG_ABSTIME
    RF_cmdTxHS.startTrigger.pastTrig = 1;
    RF_cmdTxHS.startTime = 0;

    //---START  //copy from ZhuYanjie
//  RF_cmdTxHS.pNextOp=(rfc_radioOp_t *)&RF_cmdRxHS;
    RF_cmdTxHS.pktConf.bCheckQAtEnd=0;
    RF_cmdTxHS.pktConf.bFsOff=0;
    RF_cmdTxHS.pktConf.bUseCrc=1;
    RF_cmdTxHS.pktConf.bVarLen=1;
    RF_cmdTxHS.condition.rule=COND_STOP_ON_FALSE;
	//----END   //copy from ZhuYanjie

    //smartrf_settings_predefined.c  //-10, 0, 1, 2, 3.....11, 12, 13((12.5 dBm rounded to integer),   14('14' requires CCFG_FORCE_VDDR_HH= 1)
    RF_cmdRadioSetup_hsm.txPower = RF_TxPowerTable_findValue((RF_TxPowerTable_Entry *)RF_PROP_txPowerTable, N_dBm).rawValue;
    {
        uint8_t i = 0;
        do
        {
            if ((pOverrides_fsk[i] & 0x0000FFFF) ==  0x000088A3)
            {   //RangeExtender_Dis                       //RangeExtender_En,  868M                    //RangeExtender_En,  915M  
                pOverrides_fsk[i] = (uint32_t)0x00FB88A3; //pOverrides_fsk[i] = (uint32_t)0x000188A3;  //pOverrides_fsk[i] = (uint32_t)0x000388A3;
            }
        } while ((pOverrides_fsk[i++] != 0xFFFFFFFF));

        i = 0;
        do
        {
            if ((pOverrides_lrm[i] & 0x0000FFFF) ==  0x000088A3)
            {   //RangeExtender_Dis                       //RangeExtender_En,  868M                    //RangeExtender_En,  915M  
                pOverrides_lrm[i] = (uint32_t)0x00FB88A3; //pOverrides_lrm[i] = (uint32_t)0x000188A3;  //pOverrides_lrm[i] = (uint32_t)0x000388A3;
            }
        } while ((pOverrides_lrm[i++] != 0xFFFFFFFF));

        i = 0;
        do
        {
            if ((pOverrides_sl_lr[i] & 0x0000FFFF) ==  0x000088A3)
            {   //RangeExtender_Dis                         //RangeExtender_En,  868M                    //RangeExtender_En,  915M  
                pOverrides_sl_lr[i] = (uint32_t)0x00FB88A3; //pOverrides_sl_lr[i] = (uint32_t)0x000188A3;//pOverrides_sl_lr[i] = (uint32_t)0x000388A3;
            }
        } while ((pOverrides_sl_lr[i++] != 0xFFFFFFFF));

        i = 0;
        do
        {
            if ((pOverrides_ook[i] & 0x0000FFFF) ==  0x000088A3)
            {
                pOverrides_ook[i] = (uint32_t)0x00FB88A3;
            }
        } while ((pOverrides_ook[i++] != 0xFFFFFFFF));
    }

    //RfSetup_Hsm   //4Mbps
    rfHandle = RF_open(&rfObject, &RF_prop_hsm, (RF_RadioSetup*)&RF_cmdRadioSetup_hsm, &rfParams);
    
    // Set the frequency
    RF_cmdFs_preDef.frequency = freq_MHz;  //0x393, 915M     //0x364,  868M    //915, 903, 907, 911, 919, 923, 927
    RF_cmdFs_preDef.fractFreq = 0; //fractFreq
    RF_runCmd(rfHandle, (RF_Op*)&RF_cmdFs_preDef, RF_PriorityNormal, NULL, 0);
    
}

void  RF_tran_op (uint8_t sn8b,  uint16_t  pkg_Len,  uint8_t  *ptr)
{
    tx_currentDataEntry = (rfc_dataEntryGeneral_t*)&txDataEntryBuffer;
    pPacket = &tx_currentDataEntry->data;

    pPacket[0] = 0;
    pPacket[1] = sn8b;
    tx_currentDataEntry->length= (pkg_Len +2) ;
    memcpy( &pPacket[2],  ptr, pkg_Len );

    cmdHandle = RF_postCmd(rfHandle, (RF_Op*)&RF_cmdTxHS, RF_PriorityNormal, &tx_callback, 0);

    delta_txTrig_txOK++;
    
    if(pkg_Len< 100) wait_WLpkg_tran( 1 );
    else             wait_WLpkg_tran( 6 ); //1022-bytes, wl_pkg tran,  3.4 Mbps
}

void  RF_tran_close ()
{
    RF_close(rfHandle);
}

void tx_callback(RF_Handle h, RF_CmdHandle ch, RF_EventMask e)
{
    cur_RF_EventMask = e ;
    if(e & RF_EventLastCmdDone)
    {
        if(delta_txTrig_txOK>0)
            delta_txTrig_txOK--;
    }
}
/* HIGHSPEED RX FRAME (HSM)
 * +-------------------------------------------------------------+
 * |_LEN_|_____________PAYLOAD(sz)__________|_TIMESTAMP_|_STATUS_|
 * |     |_SERIAL_|__________DATA___________|           |        |
 * |2B   |2B      | Upto 252B               | 4B        | 1B     |
 * +-------------------------------------------------------------+
 * Note that HSM mode can transfer up to 4KB of payload but are hard-coded for a maximum of 254B in this example
#define RX_FRAME_HSM_OFFSET_LEN            0
#define RX_FRAME_HSM_OFFSET_SERIAL         2
#define RX_FRAME_HSM_OFFSET_DATA           4
#define RX_FRAME_HSM_OFFSET_TIMESTAMP(sz)  (RX_FRAME_HSM_OFFSET_SERIAL + sz)   */











void  wait_WLpkg_tran( uint8_t  N_ms )
{
    Timer_start();
            cnt_N_ms= 0;
    while( (cnt_N_ms< N_ms) && (delta_txTrig_txOK>0) ) {}
    Timer_stop();
}

void *mainThread(void *arg0)
{
    uint8_t  buf[16];
    uint8_t  curPat;

    //.............
    //other code
    //.............
    while(1) {
        usleep( GAP_500ms );
        
        //------------//------------//------------//------------//------------
        // WL_tran
        //------------//------------//------------//------------//------------
        RF_tran_init  ( curWL_freq,  curWL_dBm );
        for(curPat=0; curPat<5; curPat++) { //
            get_sensor_info ( &buf[0] ) ;
            RF_tran_op ( curPat,  16,  &buf[0] );  //2+16
        }
        RF_tran_close ( );

        //------------//------------//------------//------------//------------
        // WL_recv
        //------------//------------//------------//------------//------------
        RF_recv_init ( curWL_freq );  //1.46ms
		wait_WLpkg_recv( 5 );
		//other code
        RF_recv_close ( );
    }//---- while(1) ----//
}


